%
%
%  Filaments Split Into Branches Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Filaments Functions">
%        <Item name="Split Filaments Into Branches" icon="Matlab" tooltip="Split filament into branches.">
%          <Command>MatlabXT::XTFilamentsSplitIntoBranches(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpFilaments">
%          <Item name="Split Into Branches" icon="Matlab" tooltip="Split filament into branches.">
%            <Command>MatlabXT::XTFilamentsSplitIntoBranches(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%
%   Split one filament object in different filaments. 
%   The new filaments go from the root to a terminal of the original
%       filament and they do not have branches. 
%   The number of filaments created is equal to the number of terminals
%       of the original filament.
%

function XTFilamentsSplitIntoBranches(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% get the filament
vFactory = vImarisApplication.GetFactory;
vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);

% search the filament if not previously selected
vSurpassScene = vImarisApplication.GetSurpassScene;
if ~vFactory.IsFilaments(vFilaments)        
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vFactory.IsFilaments(vDataItem)
            vFilaments = vFactory.ToFilaments(vDataItem);
            break;
        end
    end
    % did we find the filament?
    if isequal(vFilaments, [])
        msgbox('Please create some filament!');
        return;
    end
end

% get the jet colormap
vColorMap = jet;
vColorMapSize = size(vColorMap,1);
% close the figure opened by jet
close

vParent = vImarisApplication.GetFactory.CreateDataContainer;
vParent.SetName(sprintf('%s Branches', char(vFilaments.GetName)));
vFilaments.GetParent.AddChild(vParent, -1);
vFilaments.SetVisible(0);

vProgressDisplay = waitbar(0, 'Splitting filament into branches');

vCount = vFilaments.GetNumberOfFilaments;
for vFilamentIndex = 0:vCount-1
  vFilamentsXYZ = vFilaments.GetPositionsXYZ(vFilamentIndex);
  vFilamentsEdges = vFilaments.GetEdges(vFilamentIndex) + 1;
  vFilamentsRadius = vFilaments.GetRadii(vFilamentIndex);
  vFilamentsTypes = vFilaments.GetTypes(vFilamentIndex);
  vFilamentsTime = vFilaments.GetTimeIndex(vFilamentIndex);
%   vBeginningVertex = vFilaments.GetBeginningVertexIndex(vFilamentIndex) + 1;

  % find the terminals
  vNumberOfSpots = numel(vFilamentsRadius);
  vTerminals = find(FindTerminals(vNumberOfSpots, vFilamentsEdges));
  vNumberOfTerminals = numel(vTerminals);
  waitbar((vFilamentIndex+0.1)/vCount);

  for vTerminalIndex = 1:vNumberOfTerminals
    vLength = 1;
    vTerminal = vTerminals(vTerminalIndex);
    vPath = vTerminal;
    
    vFound = true;
    while vFound
        [vEdge, vSide] = find(vFilamentsEdges==vTerminal);
        
        vFound = false;
        for vNeighborIndex = 1:length(vEdge)
            vNeighbor = vFilamentsEdges(vEdge(vNeighborIndex), 3-vSide(vNeighborIndex));
            if vNeighbor<vTerminal
                vNewTerminal = vNeighbor;
                vFound = true;
           end
        end

        if vFound
            vLength = vLength + 1;
            vTerminal = vNewTerminal;
            vPath(vLength) = vTerminal;
        end
    end
    
    % terminal-root -> root-terminal
    vPath = fliplr(vPath);
        
    vNewFilaments = vImarisApplication.GetFactory.CreateFilaments;
    vNewFilaments.AddFilament(vFilamentsXYZ(vPath,:), vFilamentsRadius(vPath), ...
        vFilamentsTypes(vPath), [0:vLength-2;1:vLength-1]', vFilamentsTime);
    vNewFilaments.SetName(sprintf('%s [fil %i branch %i]', ...
      char(vFilaments.GetName), vFilamentIndex, vTerminalIndex));
    
    % computes the color
    vPercent = (vTerminalIndex-1)/max(vNumberOfTerminals-1,1)*(vColorMapSize-1)+1;
    vColor = interp1(vColorMap, vPercent);
    vNewFilaments.SetColorRGBA(round(vColor * 255) * [1; 256; 256*256]);
    vNewFilaments.SetColorSpinesRGBA(round((vColor+1)/2 * 255) * [1; 256; 256*256]);
    vParent.AddChild(vNewFilaments, -1);
    
    waitbar((vFilamentIndex+0.1+0.9*vTerminalIndex/vNumberOfTerminals)/vCount);
  end
end

close(vProgressDisplay);


function aIsTerminal = FindTerminals(aNumPoints, aEdgesElements)
aIsTerminal = true(1, aNumPoints);
vIndex(aEdgesElements) = 1:numel(aEdgesElements);
vIndex = vIndex(vIndex > 0); % holes :(
aEdgesElements(vIndex) = 0; % aEdgesElements(vIndex) = []; is slower :/
aEdgesElements = aEdgesElements(aEdgesElements > 0);
aIsTerminal(aEdgesElements) = false;
